#!/usr/bin/env python
# -*- Python -*-

"""
  \file InPort.py
  \brief InPort template class
  \date $Date: 2007/09/20 $
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2003-2005
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""
 

from omniORB import any

import OpenRTM

TIMEOUT_TICK_USEC = 10.0
USEC_PER_SEC      = 1000000.0
TIMEOUT_TICK_SEC = TIMEOUT_TICK_USEC/USEC_PER_SEC


import time
class Time:
	
	def __init__(self):
		tm = time.time()
		tm_f       = tm - int(tm)     # μФ
		self.sec   = int(tm - tm_f)   # μФ
		self.usec  = int(tm_f * USEC_PER_SEC) # sec -> usec (micro second)


class InPort:
	
	"""
	\if jp
	
	\class InPort
	
	\brief InPort 饹
 
        InPort μǤ
        󥹥RingBufferޤϡNullBufferΥ󥹥󥹤
		buff_ȤϤɬפ롣
        ǡϥե饰ˤä̤ɡɾ֤졢isNew(), getNewDataLen()
        getNewList(), getNewListReverse() Υ᥽åɤˤϥɥ󥰤뤳Ȥ
        Ǥ롣
	\endif
	"""


	def __init__(self, name, value, buffer_,
				 read_block=False, write_block=False,
				 read_timeout=0, write_timeout = 0):
		"""
		\if jp
		\brief InPort 饹󥹥ȥ饯
		
		InPort饹Υ󥹥ȥ饯
		\param name InPort ̾InPortBase:name() ˤ껲Ȥ롣
		\param value  InPort ˥Хɤ T ѿ
		\param buffer_ InPort ǻѤХåեΥ֥
		\else
		\brief A constructor.
		
		Setting channel name and registering channel value.
		\param name A name of the InPort. This name is referred by
		InPortBase::name().
		\param value A channel value related with the channel.
		\param buffer_ Buffer Object which is used in InPort 
		\endif
		"""
		self._buffer         = buffer_
		self._name           = name
		self._value          = value
		self._readBlock      = read_block
		self._readTimeout    = read_timeout
		self._writeBlock     = write_block
		self._writeTimeout   = write_timeout
		self._OnWrite        = None
		self._OnWriteConvert = None
		self._OnRead         = None
		self._OnReadConvert  = None
		self._OnOverflow     = None
		self._OnUnderflow    = None


	def __del__(self):
		"""
		\if jp
		\brief InPort饹ǥȥ饯
		
		InPort饹Υǥȥ饯
		\else
		\brief A destructor
		\endif
		"""
		pass


	def isNew(self):
		return self._buffer.isNew()
	

	def name(self):
		return self._name


	def write(self, value):
		"""
		\if jp
		\brief DataPort ͤ񤭹

		DataPort ͤ񤭹ࡣ

		- Хåե󥯥 OnWrite åȤƤ硢
			InPort ݻХåե˽񤭹 OnWrite ƤФ롣
		- InPort ݻХåեСե򸡽ФǤХåեǤꡢ
			ġ񤭹ݤ˥ХåեСե򸡽Ф硢
			Хåե󥯥 OnOverflow ƤФ롣
		- Хåե󥯥 OnWriteConvert åȤƤ硢
			Хåե񤭹߻ˡOnWriteConvert  operator()() ͤ
			Хåե˽񤭹ޤ롣
		- setWriteTimeout() ˤ񤭹߻ΥॢȤꤵƤ硢
			ॢȻ֤Хåեե֤ΤԤ
			OnOverflowåȤƤФƤӽФ롣
		\else
		\brief 
		\endif
		"""
		if self._OnWrite:
			self._OnWrite(value)

		timeout = self._writeTimeout

		tm_pre = Time()

		# blocking and timeout wait
		while self._writeBlock and self._buffer.isFull():
			if self._writeTimeout < 0:
				time.sleep(TIMEOUT_TICK_SEC)
				continue

			# timeout wait
			tm_cur = Time()

			sec  = tm_cur.sec - tm_pre.sec
			usec = tm_cur.usec - tm_pre.usec

			timeout -= (sec * USEC_PER_SEC + usec)

			if timeout < 0:
				break

			tm_pre = tm_cur
			time.sleep(TIMEOUT_TICK_USEC)

		if self._buffer.isFull() and self._OnOverflow:
			self._OnOverflow(value)
			return False

		if not self._OnWriteConvert:
			self._buffer.put(value)
		else:
			self._buffer.put(self._OnWriteConvert(value))

		return True


	def read(self):
		"""
		\if jp
		\brief DataPort ͤɤ߽Ф

		DataPort ͤɤ߽Ф

		- Хåե󥯥 OnRead åȤƤ硢
			DataPort ݻХåեɤ߽Ф OnRead ƤФ롣
		- DataPort ݻХåեե򸡽ФǤХåեǡ
			ġɤ߽Фݤ˥Хåեե򸡽Ф硢
			Хåե󥯥 OnUnderflow ƤФ롣
		- Хåե󥯥 OnReadConvert åȤƤ硢
			Хåե񤭹߻ˡOnReadConvert  operator()() ͤ
			read()ͤȤʤ롣
		- setReadTimeout() ˤɤ߽ФΥॢȤꤵƤ硢
			Хåեե֤ޤǥॢȻ֤Ԥ
			OnUnderflowåȤƤФƤӽФ
		\else
		\brief [CORBA interface] Put data on InPort
		\endif
		"""
		if self._OnRead:
			self._OnRead()

		timeout = self._readTimeout

		tm_pre = Time()

		# blocking and timeout wait
		while self._readBlock and self._buffer.isEmpty():
			if self._readTimeout < 0:
				time.sleep(TIMEOUT_TICK_SEC)
				continue

			# timeout wait
			tm_cur = Time()

			sec  = tm_cur.sec - tm_pre.sec
			usec = tm_cur.usec - tm_pre.usec
			
			timeout -= (sec * USEC_PER_SEC + usec)

			if timeout < 0:
				break

			tm_pre = tm_cur
			time.sleep(TIMEOUT_TICK_SEC)

		if self._buffer.isEmpty():
			if self._OnUnderflow:
				self._value = self._OnUnderflow()
			return self._value

		if not self._OnReadConvert:
			self._value = self._buffer.get()
			return self._value
		else:
			self._value = self._OnReadConvert(self._buffer.get())
			return self._value

		# never comes here
		return self._value


	def init(self, value):
		"""
		\if jp
		\brief InPort Υ󥰥Хåեͤ
		
		InPort Υ󥰥Хåեͤ롣
		\else
		\brief Initialize ring buffer value of InPort
		\endif
		"""
		pass
    
    
	def update(self):
		"""
		\if jp
		\brief Хɤ줿ѿself._value InPort Хåեκǿͤɤ߹
		
		Хɤ줿ǡ InPort κǿͤɤ߹ࡣ
		\else
		\brief Read data from current InPort
		\endif
		"""
		try:
			self._value = self._buffer.get()
		except:
			if self._OnUnderflow:
				self._OnUnderflow()
		return
    
    
	"""
      \if jp
     
      \brief ̤ɤοǡ
     
      \else
     
      \brief Get number of new data to be read.
     
      \endif

	def getNewDataLen(self):
		return self._buffer.new_data_len()
	"""

    
	"""
      \if jp
     
      \brief ̤ɤοǡ
     
      \else
     
      \brief Get new data to be read.
     
      \endif

	def getNewList(self):
		return self._buffer.get_new_list()

	"""

    
	"""
      \if jp
     
      \brief ̤ɤοǡս(->)Ǽ
     
      \else
     
      \brief Get new data to be read.
     
      \endif

	def getNewListReverse(self):
		return self._buffer.get_new_rlist()
	"""


	def setOnWrite(self, on_write):
		"""
		\if jp
		\brief InPort Хåե˥ǡϻΥХå
		
		InPort ĥХåե˥ǡput줿Ȥ˸ƤФ륳Хå
		֥Ȥꤹ롣ꤵ륳Хå֥Ȥ
		InPort<DataType>::OnPut饹Ѿ const DataType& 
		 void  operator() ؿƤɬפ롣
		
		struct MyOnPutCallback : public InPort<DataType> {<br>
        void operator()(const DataType data) {<br>
		<br>
        }<br>
		};<br>
		Τ褦˥Хå֥Ȥ<br>
		<br> 
		m_inport.setOnPut(new MyOnPutCallback());<br>
		Τ褦˥Хå֥Ȥ򥻥åȤ롣
		\else
		\brief Get new data to be read.
		\endif
		"""
		self._OnWrite = on_write


	def setOnWriteConvert(self, on_wconvert):
		self._OnWriteConvert = on_wconvert


	def setOnRead(self, on_read):
		self._OnRead = on_read


	def setOnReadConvert(self, on_rconvert):
		self._OnReadConvert = on_rconvert


	def setOnOverflow(self, on_overflow):
		self._OnOverflow = on_overflow


	def setOnUnderflow(self, on_underflow):
		self._OnUnderflow = on_underflow


	def getPortDataType(self):
		"""
		\brief ǡΤΥ᥽å
		ǡη̾뤿ᡢInPortCorbaProviderƤФ롣
		added by kurihara
		"""
		val = any.to_any(self._value)
		return str(val.typecode().name())
