#!/usr/bin/env python
# -*- Python -*- 

"""
  \file NVUtil.py
  \brief NameValue and NVList utility functions
  \date $Date: 2007/09/11$
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2006
      Noriaki Ando
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""

from omniORB import any

import OpenRTM
import SDOPackage, SDOPackage__POA


def newNV(name, value):
	"""
	\if jp
	\brief NameValue 

	Υڥ졼NameValue롣
	CORBA::Char, CORBA::Boolean, CORBA::Octet ϺǤʤ
	ͤ newNVChar(), newNVBool(), newNVOctet() Ǻ롣
	\param name NameValue  name
	\param value NameValue  value
	\return NameValue
	\else
	\brief Create NameVale

	This operation creates NameVale.
	CORBA::Char, CORBA::Boolean, CORBA::Octet creation is not supported.
	These type of NameValue should be created by using 
	newNVChar(), newNVBool(), newNVOctet() functions.
	\param name name of NameValue
	\param value value of NameValue
	\return NameValue
	\endif
	"""
	try:
		any_val = any.to_any(value)
	except:
		print "ERROR  NVUtil.newNV : Can't convert to any. ", type(value)
		raise

		
	nv = SDOPackage.NameValue(name, any_val)
	return nv


def newNVBool(name, value):
	"""
	\if jp

	\brief value  CORBA::Boolean  NameValue 

	Υڥ졼f value  CORBA::Boolean  NameValue롣

	\param name NameValue  name
	\param value NameValue  value
	\return NameValue

	\else

	\brief Create CORBA::Boolean value type NameVale

	This operation creates CORBA::Boolean value type NameVale.

	\param name name of NameValue
	\param value value of NameValue
	\return NameValue

	\endif
	"""
	return newNV(name, value)


def newNVOctet(name, value):
	"""
	\if jp

	\brief value  CORBA::Octet  NameValue 

	Υڥ졼f value  CORBA::Octet  NameValue롣

	\param name NameValue  name
	\param value NameValue  value
	\return NameValue

	\else

	\brief Create CORBA::Octet value type NameVale

	This operation creates CORBA::Octet value type NameVale.

	\param name name of NameValue
	\param value value of NameValue
	\return NameValue

	\endif
	"""
	return newNV(name, value)


def newNVAny(name, value):
	return newNV(name, value)


def copyFromProperties(nv, prop):
	"""
	\if jp

	\brief Properties  NVList إԡ

	Υڥ졼 Properties  NVList إԡ롣
	NVList  value  CORBA::string Ȥƥԡ롣

	\param nv Properties ͤǼ NVList
	\param prop ԡ Properties

	\else

	\brief Copy to NVList from Proeprties

	This operation copies Properties to NVList.
	Created NVList's values are CORBA::string.

	\param nv NVList to store Properties values
	\param prop Properties that is copies from

	\endif
	"""
	keys = prop.propertyNames()
	keys_len = len(keys)
	nv_len = len(nv)
	if nv_len > 0:
		for i in range(nv_len):
			del nv[-1]

	for i in range(keys_len):
		nv.append(newNV(keys[i], prop.getProperty(keys[i])))


def copyToProperties(prop, nvlist):
	for nv in nvlist:
		try:
			val = str(any.from_any(nv.value, keep_structs=True))
			prop.setProperty(str(nv.name),val)
		except:
			pass


class to_prop:
	def __init__(self):
		self._prop = OpenRTM.Properties()
		
	def __call__(self, nv):
		self._prop.setProperty(nv.name, nv.value)


def toProperties(nv):
	p = OpenRTM.CORBA_SeqUtil.for_each(nv, to_prop())
	return p._prop


class nv_find:
	"""
	\if jp

	\brief NVList  name ǻꤵ줿 value ֤

	Υڥ졼 name ǻꤵ줿 value  Any ֤

	\param nv оݤ NVList
	\param name ̾

	\else

	\brief Get value in NVList specified by name

	This operation returns Any type of value specified by name.
	Created NVList's values are CORBA::string.

	\param nv NVList to be searched
	\param prop name to seartch in NVList

	\endif
	"""
	def __init__(self, name):
		self._name = name

	def __call__(self, nv):
		return str(self._name) == str(nv.name)
	

def find(nv, name):
	index = OpenRTM.CORBA_SeqUtil.find(nv, nv_find(name))

	if index < 0:
		raise "Not found."

	return nv[index].value


def find_index(nv, name):
	return OpenRTM.CORBA_SeqUtil.find(nv, nv_find(name))


def isString(nv, name):
	"""
	\if jp

	\brief ꤵ줿 name  value η string Ǥ뤫ɤ

	Υڥ졼 name ǻꤵ줿 value η CORBA::string
	ɤ bool ֤ͤ

	\param nv оݤ NVList
	\param name ̾

	\else

	\brief Whether the name of NVList's value is CORBA::string

	This operation returns boolean value whether the name of NVList's value
	is CORBA::string.

	\param nv NVList to be searched
	\param prop name to seartch in NVList

	\endif
	"""
	try:
		value = find(nv, name)
		val = any.from_any(value, keep_structs=True)
		return type(val) == str
	except:
		return False
	

def isStringValue(nv, name, value):
	if isString(nv, name):
		if toString(nv, name) == value:
			return True
	return False


def toString(nv, name):
	"""
	\if jp

	\brief ꤵ줿 name  NVList  string Ȥ֤

	Υڥ졼 name ǻꤵ줿 NVList ͤ string ֤
	⤷name ǻꤷ value ͤ CORBA::string ǤʤС
	ʸstring֤

	\param nv оݤ NVList
	\param name ̾
	\return name бͤstring

	\else

	\brief Get string value in NVList specified by name

	This operation returns string value in NVList specified by name.
	If the value in NVList specified by name is not CORBA::string type
	this operation returns empty string value.

	\param nv NVList to be searched
	\param name name to to serach
	\return string value named by name

	\endif
	"""
	str_value = ""
	try:
		ret_value = find(nv, name)
		val = any.from_any(ret_value, keep_structs=True)
		if type(val) == str:
			str_value = val
	except:
		pass
	
	return str_value


def appendStringValue(nv, name, value):
	index = find_index(nv, name)

	if index >= 0:
		tmp_str = nv[index].value

		values = OpenRTM.split(tmp_str,",")
		find_flag = False
		for val in values:
			if val == value:
				find_flag = True

		if not find_flag:
			tmp_str += ", "
			tmp_str += value
			nv[index].value = tmp_str
	else:
		OpenRTM.CORBA_SeqUtil.push_back(nv, newNV(name, value))

	return True
			

def append(dest, src):
	for i in range(len(src)):
		OpenRTM.CORBA_SeqUtil.push_back(dest, src[i])


def dump(nv):
	for i in range(len(nv)):
		if type(nv[i].value) == str:
			print nv[i].name, ": ", nv[i].value
		else:
			print nv[i].name, ": not a string value"
