#!/usr/bin/env python
# -*- Python -*-

"""
  \file PortBase.py
  \brief RTC's Port base class
  \date $Date: 2007/09/18 $
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2006
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""


import threading

import OpenRTM
import RTC, RTC__POA


class ScopedLock:
	def __init__(self, mutex):
		self.mutex = mutex
		self.mutex.acquire()

	def __del__(self):
		self.mutex.release()



class PortBase(RTC__POA.Port):
	"""
	\if jp
	\class PortBase
	\brief Port δ쥯饹

	RTC::Port δȤʤ륯饹
	RTC::Port Ϥۤ UML Port γǰѾƤꡢۤƱΤΤȤߤʤ
	ȤǤ롣RT ݡͥȤΥ󥻥ץȤˤƤϡ
	Port ϥݡͥȤ°ݡͥȤ¾ΥݡͥȤߺ
	ԤǤꡢ̾ĤΥ󥿡եȴϢդ롣
	ݡͥȤ Port ̤ƳФ󥿡ե󶡤ޤ׵
	뤳ȤǤPortϤ³ô
	<p>
	Port ζݥ饹ϡ̾ RT ݡͥȥ󥹥Ʊ
	졢󶡡׵ᥤ󥿡եϿ塢RT ݡͥȤ
	Ͽ졢饢ǽ Port ȤƵǽ뤳ȤꤷƤ롣
	<p>
	RTC::Port  CORBA 󥿡եȤưʲΥڥ졼󶡤롣

	- get_port_profile()
	- get_connector_profiles()
	- get_connector_profile()
	- connect()
	- notify_connect()
	- disconnect()
	- notify_disconnect()
	- disconnect_all()

	Υ饹ǤϡΥڥ졼μ󶡤롣
	<p>
	Υڥ졼Τget_port_profile(), get_connector_profiles(),
	get_connector_profile(), connect(), disconnect(), disconnect_all() ϡ
	֥饹ˤä˿񤤤ѹɬפʤᡢС饤
	뤳ȤϿ侩ʤ
	<p>
	notify_connect(), notify_disconnect() ˤĤƤϡ֥饹󶡡׵
	륤󥿡եμ˱ơ񤤤ѹɬפ
	⤷ʤľܥС饤ɤ뤳ȤϿ侩줺
	Ҥ notify_connect(), notify_disconnect() ιˤƤҤ٤̤
	δؿ˴Ϣ protected ؿ򥪡С饤ɤ뤳Ȥˤ
	񤤤ѹ뤳Ȥ侩롣

	\else


	\endif
	"""


	def __init__(self, name=None):
		"""
		\if jp
		\brief 󥹥ȥ饯

		PortBase Υ󥹥ȥ饯 Port ̾ name ˼Ԥ
		ƱˡʬȤ CORBA Object ȤƳȤ PortProfile
		 port_ref ˼ȤΥ֥ȥե󥹤Ǽ롣

		\param name Port ̾

		\else

		\brief Constructor

		The constructor of the ProtBase class is given the name of this Port
		and initialized. At the same time, the PortBase activates itself
		as CORBA object and stores its object reference to the PortProfile's 
		port_ref member.

		\param name The name of Port 

		\endif
		"""
		self._profile = RTC.PortProfile("", [], RTC.Port._nil, [], RTC.RTObject._nil,[])
		
		if name == None:
			self._profile.name = ""
		else:
			self._profile.name = name
			
		self._objref = self._this()
		self._profile.port_ref = self._objref
		self._profile.owner = RTC.RTObject._nil
		self._profile_mutex = threading.RLock()


	def __del__(self):
		"""
		\if jp
		\brief ǥȥ饯
		\else
		\brief Destructor
		\endif
		"""
		pass


	def get_port_profile(self):
		"""
		\if jp

		\brief [CORBA interface] PortProfile

		PortݻPortProfile֤
		PortProfile ¤ΤϰʲΥСġ

		- name              [string ] Port ̾
		- interfaces        [PortInterfaceProfileList ] Port ݻ
			PortInterfaceProfile Υ
		- port_ref          [Port Object ] Port ȤΥ֥ȥե
		- connector_profile [ConnectorProfileList ] Port ݻ
			ConnectorProfile Υ
		- owner             [RTObject Object ]  Port ͭ
			RTObjectΥե
		- properties        [NVList ] ¾Υץѥƥ

		\return  Port  PortProfile

		\else

		\brief [CORBA interface] Get the PortProfile of the Port

		This operation returns the PortProfile of the Port.
		PortProfile struct has the following members,

		- name              [string ] The name of the Port.
		- interfaces        [PortInterfaceProfileList ] The sequence of 
			PortInterfaceProfile owned by the Port
		- port_ref          [Port Object] The object reference of the Port.
		- connector_profile [ConnectorProfileList ] The sequence of 
			ConnectorProfile owned by the Port.
		- owner             [RTObject Object] The object reference of 
			RTObject that is owner of the Port.
		- properties        [NVList] The other properties.

		\return the PortProfile of the Port

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		prof = RTC.PortProfile(self._profile.name,
							   self._profile.interfaces,
							   self._profile.port_ref,
							   self._profile.connector_profiles,
							   self._profile.owner,
							   self._profile.properties)

		return prof


	def get_connector_profiles(self):
		"""
		\if jp

		\brief [CORBA interface] ConnectorProfileList

		Portݻ ConnectorProfile  sequence ֤
		ConnectorProfile  Port ֤³ץեݻ빽¤ΤǤꡢ
		³Port֤Ǿ򴹤ԤϢ뤹٤Ƥ Port Ʊͤ
		ݻ롣
		ConnectorProfile ϰʲΥСݻƤ롣

		- name         [string ] Υͥ̾
		- connector_id [string ] ˡID
		- ports        [Port sequnce] Υͥ˴Ϣ Port Υ֥
			ե󥹤Υ󥹡
		- properties   [NVList ] ¾Υץѥƥ

		\return  Port  ConnectorProfile

		\else

		\brief [CORBA interface] Get the ConnectorProfileList of the Port

		This operation returns a list of the ConnectorProfiles of the Port.
		ConnectorProfile includes the connection information that describes 
		relation between (among) Ports, and Ports exchange the ConnectionProfile
		on connection process and hold the same information in each Port.
		ConnectionProfile has the following members,

		- name         [string] The name of the connection.
		- connector_id [string] Unique identifier.
		- ports        [Port sequnce] The sequence of Port's object reference
			that are related the connection.
		- properties   [NVList] The other properties.

		\return the ConnectorProfileList of the Port

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		return self._profile.connector_profiles


	def get_connector_profile(self, connector_id):
		"""
		\if jp

		\brief [CORBA interface] ConnectorProfile 

		connector_id ǻꤵ줿 ConnectorProfile ֤

		\param connector_id ConnectorProfile  ID
		\return connector_id  ConnectorProfile

		\else

		\brief [CORBA interface] Get the ConnectorProfile

		This operation returns the ConnectorProfiles specified connector_id.

		\param connector_id ID of the ConnectorProfile
		\return the ConnectorProfile identified by the connector_id

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		index = OpenRTM.CORBA_SeqUtil.find(self._profile.connector_profiles,
										   self.find_conn_id(connector_id))
		if index < 0:
			conn_prof = RTC.ConnectorProfile("","",[],[])
			return conn_prof

		conn_prof = RTC.ConnectorProfile(self._profile.connector_profiles[index].name,
										 self._profile.connector_profiles[index].connector_id,
										 self._profile.connector_profiles[index].ports,
										 self._profile.connector_profiles[index].properties)
		return conn_prof


	def connect(self, connector_profile):
		"""
		\if jp

		\brief [CORBA interface] Port ³Ԥ

		Ϳ줿 ConnectoionProfile ˤäơPort֤³Ω롣
		ץꥱץ¦ϡĤΥݡͥȤʣ
		Port ³硢Ŭڤͤ򥻥åȤ ConnectorProfile 
		connect() ΰȤͿƥ뤹뤳ȤˤꡢϢ Port 
		³Ω롣

		connect() Ϳ ConnectorProfile ΥСΤname, ports, 
		(properties) СФƥǡ򥻥åȤʤФʤʤ

		\param connector_profile ConnectorProfile
		\return ReturnCode_t ڥ졼Υ꥿󥳡

		\else

		\brief [CORBA interface] Connect the Port

		This operation establishes connection according to the given 
		ConnectionProfile inforamtion. 
		Application programs, which is going to establish the connection 
		among Ports owned by RT-Components, have to set valid values to the 
		ConnectorProfile and give it to the argument of connect() operation.

		name, ports, (properties) members of ConnectorProfile should be set
		valid values before giving to the argument of connect() operation.

		\param connector_profile The ConnectorProfile.
		\return ReturnCode_t The return code of this operation.

		\endif
		"""
		if self.isEmptyId(connector_profile):
			self.setUUID(connector_profile)
			assert(not self.isExistingConnId(connector_profile.connector_id))

		try:
			retval,connector_profile = connector_profile.ports[0].notify_connect(connector_profile)
			return (retval, connector_profile)
			#return connector_profile.ports[0].notify_connect(connector_profile)
		except:
			return (RTC.BAD_PARAMETER, connector_profile)

		return (RTC.RTC_ERROR, connector_profile)


	def notify_connect(self, connector_profile):
		"""
		\if jp

		\brief [CORBA interface] Port ³ΤԤ

		Υڥ졼ϡPort֤³ԤݤˡPort֤Ū
		ƤФ륪ڥ졼Ǥ롣

		\param connector_profile ConnectorProfile
		\return ReturnCode_t ڥ졼Υ꥿󥳡

		\else

		\brief [CORBA interface] Notify the Ports connection

		This operation is invoked between Ports internally when the connection
		is established.

		\param connector_profile The ConnectorProfile.
		\return ReturnCode_t The return code of this operation.

		\endif
		"""
		# publish owned interface information to the ConnectorProfile
		retval = self.publishInterfaces(connector_profile)

		if retval != RTC.RTC_OK:
			return (retval, connector_profile)

		# call notify_connect() of the next Port
		retval, connector_profile = self.connectNext(connector_profile)
		if retval != RTC.RTC_OK:
			return (retval, connector_profile)

		# subscribe interface from the ConnectorProfile's information
		retval = self.subscribeInterfaces(connector_profile)
		if retval != RTC.RTC_OK:
			#cleanup this connection for downstream ports
			self.notify_disconnect(connector_profile.connector_id)
			return (retval, connector_profile)

		# update ConnectorProfile
		index = self.findConnProfileIndex(connector_profile.connector_id)
		if index < 0:
			OpenRTM.CORBA_SeqUtil.push_back(self._profile.connector_profiles,
											connector_profile)
		else:
			self._profile.connector_profiles[index] = connector_profile

		return (retval, connector_profile)


	def disconnect(self, connector_id):
		"""
		\if jp

		\brief [CORBA interface] Port ³

		Υڥ졼³Ω³ФͿ connector_id 
		бԥ Port Ȥ³롣

		\param connector_id ConnectorProfile  ID
		\return ReturnCode_t ڥ졼Υ꥿󥳡

		\else

		\brief [CORBA interface] Connect the Port

		This operation destroys connection between this port and the peer port
		according to given id that is given when the connection established.

		\param connector_id The ID of the ConnectorProfile.
		\return ReturnCode_t The return code of this operation.

		\endif
		"""
		# find connector_profile
		if not self.isExistingConnId(connector_id):
			return RTC.BAD_PARAMETER

		index = self.findConnProfileIndex(connector_id)
		prof = RTC.ConnectorProfile(self._profile.connector_profiles[index].name,
									self._profile.connector_profiles[index].connector_id,
									self._profile.connector_profiles[index].ports,
									self._profile.connector_profiles[index].properties)
		return prof.ports[0].notify_disconnect(connector_id)


	def notify_disconnect(self, connector_id):
		"""
		\if jp

		\brief [CORBA interface] Port ³ΤԤ

		Υڥ졼ϡPort֤³ԤݤˡPort֤Ū
		ƤФ륪ڥ졼Ǥ롣

		\param connector_id ConnectorProfile  ID
		\return ReturnCode_t ڥ졼Υ꥿󥳡

		\else

		\brief [CORBA interface] Notify the Ports disconnection

		This operation is invoked between Ports internally when the connection
		is destroied.

		\param connector_id The ID of the ConnectorProfile.
		\return ReturnCode_t The return code of this operation.

		\endif
		"""

		# The Port of which the reference is stored in the beginning of
		# connectorProfile's PortList is master Port.
		# The master Port has the responsibility of disconnecting all Ports.
		# The slave Ports have only responsibility of deleting its own
		# ConnectorProfile.

		# find connector_profile
		if not self.isExistingConnId(connector_id):
			return RTC.BAD_PARAMETER

		index = self.findConnProfileIndex(connector_id)
		prof = RTC.ConnectorProfile(self._profile.connector_profiles[index].name,
									self._profile.connector_profiles[index].connector_id,
									self._profile.connector_profiles[index].ports,
									self._profile.connector_profiles[index].properties)

		self.unsubscribeInterfaces(prof)
		retval = self.disconnectNext(prof)

		OpenRTM.CORBA_SeqUtil.erase(self._profile.connector_profiles, index)
    
		return retval


	def disconnect_all(self):
		"""
		\if jp

		\brief [CORBA interface] Port ³

		Υڥ졼Ϥ Port ˴ϢƤ³롣

		\return ReturnCode_t ڥ졼Υ꥿󥳡

		\else

		\brief [CORBA interface] Connect the Port

		This operation destroys all connection channels owned by the Port.

		\return ReturnCode_t The return code of this operation.

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		# disconnect all connections
		# Call disconnect() for each ConnectorProfile.
		f = OpenRTM.CORBA_SeqUtil.for_each(self._profile.connector_profiles,
										   self.disconnect_all_func(self))
		return f.return_code



	#============================================================
	# Local operations
	#============================================================

	def setName(self, name):
		"""
		\if jp
		\brief Port ̾ꤹ

		Port ̾ꤹ롣̾ Port ݻ PortProfile.name
		ȿǤ롣

		\param name Port ̾

		\else
		\brief Set the name of this Port

		This operation sets the name of this Port. The given Port's name is
		applied to Port's PortProfile.name.

		\param name The name of this Port.

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		self._profile.name = name


	def getProfile(self):
		"""
		\if jp
		\brief PortProfile

		Portݻ PortProfile  const Ȥ֤

		\return  Port  PortProfile

		\else
		\brief Get the PortProfile of the Port

		This operation returns const reference of the PortProfile.

		\return the PortProfile of the Port

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		return self._profile


	def setPortRef(self, port_ref):
		"""
		\if jp

		\brief Port Υ֥ȻȤꤹ

		Υڥ졼 Port  PortProfile ˤ Port Ȥ
		֥ȻȤꤹ롣

		\param  Port Υ֥Ȼ

		\else

		\brief Set the object reference of this Port

		This operation sets the object reference itself
		to the Port's PortProfile.

		\param The object reference of this Port.

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		self._profile.port_ref = port_ref


	def getPortRef(self):
		"""
		\if jp

		\brief Port Υ֥ȻȤ

		Υڥ졼 Port  PortProfile ݻƤ
		 Port ȤΥ֥ȻȤ롣

		\return  Port Υ֥Ȼ

		\else

		\brief Get the object reference of this Port

		This operation returns the object reference
		that is stored in the Port's PortProfile.

		\return The object reference of this Port.

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		return self._profile.port_ref


	def setOwner(self, owner):
		"""
		\if jp

		\brief Port  owner  RTObject ꤹ

		Υڥ졼 Port  PortProfile.owner ꤹ롣

		\param owner  Port ͭ RTObject λ

		\else

		\brief Set the owner RTObject of the Port

		This operation sets the owner RTObject of this Port.

		\param owner The owner RTObject's reference of this Port

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)
		self._profile.owner = owner


	"""
      \if jp
     
      \brief Interface 
     
      Υڥ졼ϡnotify_connect() 󥹤λϤ˥
      貾۴ؿǤ롣
      notify_connect() Ǥϡ
     
      - publishInterfaces()
      - connectNext()
      - subscribeInterfaces()
      - updateConnectorProfile()
     
      ν protected ؿ뤵³Ԥ롣
      <br>
       Port ǤϤΥڥ졼򥪡С饤ɤȤ
      Ϳ줿 ConnectorProfile ˽Ԥѥ᡼Ŭ
      ǤСRteurnCode_t Υ顼ɤ֤
      ̾ publishInterafaces() ˤƤϡ Port °
      󥿡ե˴ؤ ConnectorProfile ФŬڤꤷ
      ¾ Port ΤʤФʤʤ
      <br>
      ޤδؿ뤵ʳǤϡ¾ Port  Interface ˴ؤ
      Ϥ٤ƴޤޤƤʤΤǡ¾ Port  Interface 
       subscribeInterfaces() ǹԤ٤Ǥ롣
      <br>
      Υڥ졼ϡ connector_id ФƤ³
      ¸ connector_id ФƤϹŬڤ˹Ԥɬפ롣
     
      \param connector_profile ³˴ؤץե
      \return ReturnCode_t Υ꥿󥳡
     
      \else
     
      \brief Publish interface information
     
      This operation is pure virutal method that would be called at the
      beginning of the notify_connect() process sequence.
      In the notify_connect(), the following methods would be called in order.
     
      - publishInterfaces()
      - connectNext()
      - subscribeInterfaces()
      - updateConnectorProfile() 
     
      In the concrete Port, this method should be overridden. This method
      processes the given ConnectorProfile argument and if the given parameter
      is invalid, it would return error code of ReturnCode_t.
      Usually, publishInterfaces() method should set interfaces information
      owned by this Port, and publish it to the other Ports.
      <br>
      When this method is called, other Ports' interfaces information may not
      be completed. Therefore, the process to obtain other Port's interfaces
      information should be done in the subscribeInterfaces() method.
      <br>
      This operation should create the new connection for the new
      connector_id, and should update the connection for the existing
      connection_id.
     
      \param connector_profile The connection profile information
      \return The return code of ReturnCode_t type.
     
      \endif

	def publishInterfaces(self, connector_profile):
		pass
	"""    


	def connectNext(self, connector_profile):
		"""
		\if jp

		\brief  Port Ф notify_connect() 򥳡뤹

		ConnectorProfile  port_ref ˳ǼƤ Port Υ֥
		ե󥹤Υ󥹤椫顢Ȥ Port μ Port Ф
		notify_connect() 򥳡뤹롣

		\param connector_profile ³˴ؤץե
		\return ReturnCode_t Υ꥿󥳡

		\else

		\brief Call notify_connect() of the next Port

		This operation calls the notify_connect() of the next Port's 
		that stored in ConnectorProfile's port_ref sequence.

		\param connector_profile The connection profile information
		\return The return code of ReturnCode_t type.

		\endif
		"""
		index = OpenRTM.CORBA_SeqUtil.find(connector_profile.ports,
										   self.find_port_ref(self._profile.port_ref))
		if index < 0:
			return RTC.BAD_PARAMETER, connector_profile

		index += 1
		if index < len(connector_profile.ports):
			p = connector_profile.ports[index]
			return p.notify_connect(connector_profile)

		return RTC.RTC_OK, connector_profile


	def disconnectNext(self, connector_profile):
		"""
		\if jp

		\brief  Port Ф notify_disconnect() 򥳡뤹

		ConnectorProfile  port_ref ˳ǼƤ Port Υ֥
		ե󥹤Υ󥹤椫顢Ȥ Port μ Port Ф
		notify_disconnect() 򥳡뤹롣

		\param connector_profile ³˴ؤץե
		\return ReturnCode_t Υ꥿󥳡

		\else

		\brief Call notify_disconnect() of the next Port

		This operation calls the notify_disconnect() of the next Port's 
		that stored in ConnectorProfile's port_ref sequence.

		\param connector_profile The connection profile information
		\return The return code of ReturnCode_t type.

		\endif
		"""
		index = OpenRTM.CORBA_SeqUtil.find(connector_profile.ports,
										   self.find_port_ref(self._profile.port_ref))
		if index < 0:
			return RTC.BAD_PARAMETER

		index += 1
		
		if index < len(connector_profile.ports):
			p = connector_profile.ports[index]
			return p.notify_disconnect(connector_profile.connector_id)

		self.unsubscribeInterfaces(connector_profile)
		return RTC.RTC_OK


	"""
      \if jp
     
      \brief Interface 
     
      Υڥ졼ϡnotify_connect() 󥹤֤˥
      貾۴ؿǤ롣
      notify_connect() Ǥϡ
     
      - publishInterfaces()
      - connectNext()
      - subscribeInterfaces()
      - updateConnectorProfile()
     
      ν protected ؿ뤵³Ԥ롣
      <br>
       Port ǤϤΥڥ졼򥪡С饤ɤȤ
      Ϳ줿 ConnectorProfile ˽Ԥѥ᡼Ŭ
      ǤСRteurnCode_t Υ顼ɤ֤
       ConnectorProfile ˤ¾ Port  Interface ˴ؤ
      ƴޤޤƤ롣
      ̾ subscribeInterafaces() ˤƤϡ Port Ѥ
      Interface ˴ؤ׵¦Υ󥿡եФ
      ꤷʤФʤʤ
      <br>
      Υڥ졼ϡ connector_id ФƤ³
      ¸ connector_id ФƤϹŬڤ˹Ԥɬפ롣
     
      \param connector_profile ³˴ؤץե
      \return ReturnCode_t Υ꥿󥳡
     
      \else
     
      \brief Publish interface information
     
      This operation is pure virutal method that would be called at the
      mid-flow of the notify_connect() process sequence.
      In the notify_connect(), the following methods would be called in order.
     
      - publishInterfaces()
      - connectNext()
      - subscribeInterfaces()
      - updateConnectorProfile()
     
      In the concrete Port, this method should be overridden. This method
      processes the given ConnectorProfile argument and if the given parameter
      is invalid, it would return error code of ReturnCode_t.
      The given argument ConnectorProfile includes all the interfaces
      information in it.
      Usually, subscribeInterafaces() method obtains information of interfaces
      from ConnectorProfile, and should set it to the interfaces that require
      them.
      <br>
      This operation should create the new connection for the new
      connector_id, and should update the connection for the existing
      connection_id.
     
      \param connector_profile The connection profile information
      \return The return code of ReturnCode_t type.
     
      \endif

	def subscribeInterfaces(self, connector_profile):
		pass
	"""    
    
	"""
      \if jp
     
      \brief Interface ³
     
      Υڥ졼ϡnotify_disconnect() 󥹤ν˥
      貾۴ؿǤ롣
      notify_disconnect() Ǥϡ
      - disconnectNext()
      - unsubscribeInterfaces()
      - eraseConnectorProfile()
      ν protected ؿ뤵³Ԥ롣
      <br>
       Port ǤϤΥڥ졼򥪡С饤ɤȤ
      Ϳ줿 ConnectorProfile ˽³Ԥ
     
      \param connector_profile ³˴ؤץե
     
      \else
     
      \brief Disconnect interface connection
     
      This operation is pure virutal method that would be called at the
      end of the notify_disconnect() process sequence.
      In the notify_disconnect(), the following methods would be called.
      - disconnectNext()
      - unsubscribeInterfaces()
      - eraseConnectorProfile() 
       <br>
      In the concrete Port, this method should be overridden. This method
      processes the given ConnectorProfile argument and disconnect interface
      connection.
     
      \param connector_profile The connection profile information
     
      \endif

	def unsubscribeInterfaces(self, connector_profile):
		pass
	"""


	def isEmptyId(self, connector_profile):
		"""
		\if jp

		\brief ConnectorProfile  connector_id եɤɤȽ

		\return Ϳ줿 ConnectorProfile  connector_id ǤС
			trueǤʤ false ֤

		\else

		\brief Whether connector_id of ConnectorProfile is empty

		\return If the given ConnectorProfile's connector_id is empty string,
			it returns true.

		\endif
		"""
		return connector_profile.connector_id == ""


	def getUUID(self):
		"""
		\if jp

		\brief UUID

		Υڥ졼 UUID 롣

		\return uuid

		\else

		\brief Get the UUID

		This operation generates UUID.

		\return uuid

		\endif
		"""
		return str(OpenRTM.uuid1())


	def setUUID(self, connector_profile):
		"""
		\if jp

		\brief UUID ConnectorProfile ˥åȤ

		Υڥ졼 UUID ConnectorProfile ˥åȤ롣

		\param connector_profile connector_id 򥻥åȤ ConnectorProfile

		\else

		\brief Create and set the UUID to the ConnectorProfile

		This operation generates and set UUID to the ConnectorProfile.

		\param connector_profile ConnectorProfile to be set connector_id

		\endif
		"""
		connector_profile.connector_id = self.getUUID()
		assert(connector_profile.connector_id != "")


	def isExistingConnId(self, id_):
		"""
		\if jp

		\brief id ¸ ConnectorProfile ΤΤɤȽꤹ

		Υڥ졼Ϳ줿 ID ¸ ConnectorProfile Υꥹ
		¸ߤ뤫ɤȽꤹ롣

		\param id Ƚꤹ connector_id

		\else

		\brief Whether the given id exists in stored ConnectorProfiles

		This operation returns boolean whether the given id exists in 
		the Port's ConnectorProfiles.

		\param id connector_id to be find in Port's ConnectorProfiles

		\endif
		"""
		return OpenRTM.CORBA_SeqUtil.find(self._profile.connector_profiles,
										  self.find_conn_id(id_)) >= 0


	def findConnProfile(self, id_):
		"""
		\if jp

		\brief id  ConnectorProfile õ

		Υڥ졼Ϳ줿 ID  ConnectorProfile  Port 
		 ConnectorProfile Υꥹ椫õ
		⤷Ʊ id  ConnectorProfile ʤС ConnectorProfile
		֤롣

		\param id  connector_id
		\return connector_id  ConnectorProfile

		\else

		\brief Find ConnectorProfile with id

		This operation returns ConnectorProfile with the given id from Port's
		ConnectorProfiles' list.
		If the ConnectorProfile with connector id that is identical with the
		given id does not exist, empty ConnectorProfile is returned.

		\param id the connector_id to be searched in Port's ConnectorProfiles
		\return CoonectorProfile with connector_id

		\endif
		"""
		index = OpenRTM.CORBA_SeqUtil.find(self._profile.connector_profiles,
										   self.find_conn_id(id_))
		if index < 0 or index >= len(self._profile.connector_profiles):
			return RTC.ConnectorProfile("","",[],[])

		return self._profile.connector_profiles[index]


	def findConnProfileIndex(self, id_):
		"""
		\if jp

		\brief id  ConnectorProfile õ

		Υڥ졼Ϳ줿 ID  ConnectorProfile  Port 
		 ConnectorProfile Υꥹ椫õǥå֤
		⤷Ʊ id  ConnectorProfile ʤС-1 ֤

		\param id  connector_id
		\return Port  ConnectorProfile ꥹȤΥǥå

		\else

		\brief Find ConnectorProfile with id

		This operation returns ConnectorProfile with the given id from Port's
		ConnectorProfiles' list.
		If the ConnectorProfile with connector id that is identical with the
		given id does not exist, empty ConnectorProfile is returned.

		\param id the connector_id to be searched in Port's ConnectorProfiles
		\return The index of ConnectorProfile of the Port

		\endif
		"""
		return OpenRTM.CORBA_SeqUtil.find(self._profile.connector_profiles,
										  self.find_conn_id(id_))


	def updateConnectorProfile(self, connector_profile):
		"""
		\if jp

		\brief ConnectorProfile ɲä⤷Ϲ

		Υڥ졼Ϳ줿 ConnectorProfile Port ɲä⤷
		¸롣
		Ϳ줿 ConnectorProfile  connector_id Ʊ ID 
		ConnectorProfile ꥹȤˤʤСꥹȤɲä
		Ʊ ID ¸ߤ ConnectorProfile ¸롣

		\param coonector_profile ɲä⤷Ϲ ConnectorProfile

		\else

		\brief Append or update the ConnectorProfile list

		This operation appends or updates ConnectorProfile of the Port
		by the given ConnectorProfile.
		If the connector_id of the given ConnectorProfile does not exist
		in the Port's ConnectorProfile list, the given ConnectorProfile would be
		append to the list. If the same id exists, the list would be updated.

		\param connector_profile the ConnectorProfile to be appended or updated

		\endif
		"""
		index = OpenRTM.CORBA_SeqUtil.find(self._profile.connector_profiles,
										   self.find_conn_id(connector_profile.connector_id))

		if index < 0:
			OpenRTM.CORBA_SeqUtil.push_back(self._profile.connector_profiles,
											self.connector_profile)
		else:
			self._profile.connector_profiles[index] = connector_profile


	def eraseConnectorProfile(self, id_):
		"""
		\if jp

		\brief ConnectorProfile 

		Υڥ졼 Port  PortProfile ݻƤ
		ConnectorProfileList ΤͿ줿 id  ConnectorProfile
		롣

		\param id  ConnectorProfile  id

		\else

		\brief Delete the ConnectorProfile

		This operation deletes a ConnectorProfile specified by id from
		ConnectorProfileList owned by PortProfile of this Port.

		\param id The id of the ConnectorProfile to be deleted.

		\endif
		"""
		guard = ScopedLock(self._profile_mutex)

		index = OpenRTM.CORBA_SeqUtil.find(self._profile.connector_profiles,
										   self.find_conn_id(id_))

		if index < 0:
			return False

		OpenRTM.CORBA_SeqUtil.erase(self._profile.connector_profiles, index)

		return True


	def appendInterface(self, instance_name, type_name, pol):
		"""
		\if jp

		\brief PortInterfaceProfile  󥿡եϿ

		Υڥ졼 Port  PortProfile ΡPortInterfaceProfile
		˥󥿡եξɲä롣
		ξϡget_port_profile() ˤä PortProfile Τ
		PortInterfaceProfile ͤѹΤߤǤꡢºݤ˥󥿡ե
		󶡤׵ᤷꤹˤϡ֥饹ǡpublishInterface(),
		subscribeInterface() δؿŬڤ˥С饤ɤ󥿡ե
		󶡡׵ԤʤФʤʤ

		󥿡ե(Υ󥹥)̾ Port ǰդǤʤФʤʤ
		Ʊ̾Υ󥿡եǤϿƤ硢δؿ false 
		֤

		\param name 󥿡եΥ󥹥󥹤̾
		\param type_name 󥿡եη̾
		\param pol 󥿡ե° (RTC::PROVIDED ⤷ RTC:REQUIRED)
		\return Ʊ̾Υ󥿡եϿƤ false ֤

		\else

		\brief Append an interface to the PortInterfaceProfile

		This operation appends interface information to the PortInterfaceProfile
		that is owned by the Port.
		The given interfaces information only updates PortInterfaceProfile of
		PortProfile that is obtained through get_port_profile().
		In order to provide and require interfaces, proper functions (for
		example publishInterface(), subscribeInterface() and so on) should be
		overridden in subclasses, and these functions provide concrete interface
		connection and disconnection functionality.

		The interface (instance) name have to be unique in the Port.
		If the given interface name is identical with stored interface name,
		this function returns false.

		\param name The instance name of the interface.
		\param type_name The type name of the interface.
		\param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
		\return false would be returned if the same name is already registered.

		\endif
		"""
		index = OpenRTM.CORBA_SeqUtil.find(self._profile.interfaces,
										   self.find_interface(instance_name, pol))

		if index >= 0:
			return False

		# setup PortInterfaceProfile
		prof = RTC.PortInterfaceProfile(instance_name, type_name, pol)
		OpenRTM.CORBA_SeqUtil.push_back(self._profile.interfaces, prof)

		return True


	def deleteInterface(self, name, pol):
		"""
		\if jp

		\brief PortInterfaceProfile 饤󥿡եϿ

		Υڥ졼 Port  PortProfile ΡPortInterfaceProfile
		饤󥿡եξ롣

		\param name 󥿡եΥ󥹥󥹤̾
		\param pol 󥿡ե° (RTC::PROVIDED ⤷ RTC:REQUIRED)
		\return 󥿡եϿƤʤ false ֤

		\else

		\brief Delete an interface from the PortInterfaceProfile

		This operation deletes interface information from the
		PortInterfaceProfile that is owned by the Port.

		\param name The instance name of the interface.
		\param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
		\return false would be returned if the given name is not registered.

		\endif
		"""
		index = OpenRTM.CORBA_SeqUtil.find(self._profile.interfaces,
										   self.find_interface(name, pol))

		if index < 0:
			return False

		OpenRTM.CORBA_SeqUtil.erase(self._profile.interfaces, index)
		return True


	def addProperty(self, key, value):
		"""
		\if jp

		\brief PortProfile  properties  NameValue ͤɲä

		\param ValueType properties  value ɲäͤη
		\param key properties  name
		\param value properties  value

		\else

		\brief Add NameValue data to PortProfile's properties

		\param ValueType The type of value
		\param key The name of properties
		\param value The value of properties

		\endif
		"""
		OpenRTM.CORBA_SeqUtil.push_back(self._profile.properties,
										OpenRTM.NVUtil.newNV(key, value))


	#============================================================
	# Functor
	#============================================================

	class if_name:
		"""
		\if jp
		\brief instance_name  PortInterfaceProfile õ Functor
		\else
		\brief A functor to find a PortInterfaceProfile named instance_name
		\endif
		"""
		def __init__(self, name):
			self._name = name

		def __call__(self, prof):
			return str(self._name) == str(prof.instance_name)
    

	class find_conn_id:
		"""
		\if jp
		\brief id  ConnectorProfile õ Functor
		\else
		\brief A functor to find a ConnectorProfile named id
		\endif
		"""
		def __init__(self, id_):
			self._id = id_

		def __call__(self, cprof):
			return str(self._id) == str(cprof.connector_id)


	class find_port_ref:
		"""
		\if jp
		\brief 󥹥ȥ饯 port_ref Ʊ֥ȻȤõ Functor
		\else
		\brief A functor to find the object reference that is identical port_ref
		\endif
		"""
		def __init__(self, port_ref):
			self._port_ref = port_ref

		def __call__(self, port_ref):
			return self._port_ref._is_equivalent(port_ref)


	class connect_func:
		"""
		\if jp
		\brief Port ³Ԥ Functor
		\else
		\brief A functor to connect Ports
		\endif
		"""
		def __init__(self, p, prof):
			self._port_ref = p
			self._connector_profile = prof
			self.return_code = RTC.RTC_OK

		def __call__(self, p):
			if not self._port_ref._is_equivalent(p):
				retval = p.notify_connect(self._connector_profile)
				if retval != RTC.RTC_OK:
					self.return_code = retval


	class disconnect_func:
		"""
		\if jp
		\brief Port ³Ԥ Functor
		\else
		\brief A functor to disconnect Ports
		\endif
		"""
		def __init__(self, p, prof):
			self._port_ref = p
			self._connector_profile = prof
			self.return_code = RTC.RTC_OK
			
		def __call__(self, p):
			if not self._port_ref._is_equivalent(p):
				retval = p.disconnect(self._connector_profile.connector_id)
				if retval != RTC.RTC_OK:
					self.return_code = retval


	class disconnect_all_func:
		"""
		\if jp
		\brief Port ³Ԥ Functor
		\else
		\brief A functor to disconnect all Ports
		\endif
		"""
		def __init__(self, p):
			self.return_code = RTC.RTC_OK
			self._port = p

		def __call__(self, p):
			retval = self._port.disconnect(p.connector_id)
			if retval != RTC.RTC_OK:
				self.return_code = retval
				
	class find_interface:
		"""
		\if jp
		\brief name  polarity  interface õ Functor
		\else
		\brief A functor to find interface from name and polarity
		\endif
		"""
		def __init__(self, name, pol):
			self._name = name
			self._pol = pol

		def __call__(self, prof):
			name = prof.instance_name
			return (str(self._name) == str(name)) and (self._pol == prof.polarity)
